package cn.ibizlab.core.data.service.impl;

import cn.ibizlab.core.data.domain.DOModel;
import cn.ibizlab.core.data.domain.DOModel;
import cn.ibizlab.core.data.dto.BaseData;
import cn.ibizlab.core.data.dto.DbDataQuery;
import cn.ibizlab.core.data.filter.DOModelSearchContext;
import cn.ibizlab.core.data.mapper.DbDataMapper;
import cn.ibizlab.core.data.model.POSchema;
import cn.ibizlab.core.data.model.PojoSchema;
import cn.ibizlab.core.data.service.IDOModelService;
import cn.ibizlab.core.data.service.IDataService;
import cn.ibizlab.util.errors.BadRequestAlertException;
import cn.ibizlab.util.filter.SearchContextBase;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.ObjectUtils;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

/**
 * 实体[业务实体] 无存储服务对象接口实现
 */
@Slf4j
@Service
public class DbDataServiceImpl extends BaseDataService {


    @Autowired
    private DbDataMapper dbDataMapper;

    @Override
    public boolean create(DOModel model, String datasource, BaseData et)
    {
        Serializable key=model.getKeyValue(et,true);
        if(ObjectUtils.isEmpty(key))
            throw new BadRequestAlertException("未找到主键",model.getName(),null);
        if(dbDataMapper.insertData(model.getDefaultPOSchema(),et)==1)
            et.setAll(get(model,datasource,key));
        return true;
    }

    @Override
    public void createBatch(DOModel model, String datasource, List<BaseData> list)
    {

    }

    @Override
    public boolean update(DOModel model, String datasource, BaseData et)
    {
        Serializable key=model.getKeyValue(et,true);
        if(ObjectUtils.isEmpty(key))
            throw new BadRequestAlertException("未找到主键",model.getName(),null);
        dbDataMapper.updateData(model.getDefaultPOSchema(),et);
        et.setAll(get(model,datasource,(Serializable)key));
        return true;
    }

    @Override
    public void updateBatch(DOModel model, String datasource, List<BaseData> list)
    {

    }

    @Override
    public boolean remove(DOModel model, String datasource, Serializable key)
    {
        if(ObjectUtils.isEmpty(key))
            throw new BadRequestAlertException("未找到主键",model.getName(),null);
        BaseData et = model.newData(key);
        if(et==null)
            throw new BadRequestAlertException("未找到主键",model.getName(),null);
        dbDataMapper.removeData(model.getDefaultPOSchema(),et);
        return true;
    }

    @Override
    public void removeBatch(DOModel model, String datasource, List<Serializable> idList)
    {

    }

    @Override
    public BaseData get(DOModel model, String datasource, Serializable key)
    {
        if(ObjectUtils.isEmpty(key))
            throw new BadRequestAlertException("未找到主键",model.getName(),null);
        BaseData et = model.newData(key);
        if(et==null)
            throw new BadRequestAlertException("未找到主键",model.getName(),null);
        POSchema poSchema=model.getDefaultPOSchema();
        List<BaseData> list=dbDataMapper.getData(poSchema,et);
        if(ObjectUtils.isEmpty(list)||list.size()>1)
            throw new BadRequestAlertException("未找到数据",model.getName(),key.toString());
        if(list.size()>1)
            throw new BadRequestAlertException("数据不唯一",model.getName(),key.toString());
        return poSchema.trans(list.get(0));
    }

    @Override
    public BaseData getDraft(DOModel model, String datasource, BaseData et)
    {
        return null;
    }

    @Override
    public boolean checkKey(DOModel model, String datasource, BaseData et)
    {
        Serializable key=model.getKeyValue(et,true);
        if(ObjectUtils.isEmpty(key))
            return false;

        List<BaseData> list=dbDataMapper.getData(model.getDefaultPOSchema(),et);
        if(ObjectUtils.isEmpty(list)&&list.size()==1)
            return list.get(0).getIntegerValue("cnt",0)==1;
        return false;
    }

    @Override
    public boolean save(DOModel model, String datasource, BaseData et)
    {
        if(checkKey(model,datasource,et))
            return update(model,datasource,et);
        else
            return create(model,datasource,et);
    }

    @Override
    public void saveBatch(DOModel model, String datasource, List<BaseData> list)
    {

    }

    @Override
    public Page<BaseData> fetch(DOModel model, String dataSet, String datasource, SearchContextBase context)
    {
        return null;
    }

    @Override
    public List<BaseData> list(DOModel model, String dataSet, String datasource, SearchContextBase context)
    {
        return null;
    }


}


